import gowork
import json
import os
import base64
import platform
import pathlib


class Credentials:
    def __init__(self):
        self.__platform = '\\' if platform.system().__str__() == 'Windows' else '/'
        self.__root = self.__platform.join(gowork.__file__.split(self.__platform)[:-1] + ['safe', 'secret_keys.json'])
        self.__creds = {}
        self.__project = pathlib.Path().resolve().__str__()

    def insert(self, name: str, connector: str, credentials: dict, encode=[]):
        name = name.lower()
        connector = connector.lower()

        self.__load()
        if connector not in self.__creds.keys():
            self.__creds[connector] = {}
        if len(encode) > 0:
            credentials = self.__encode(credentials, encode)
        self.__creds[connector][name] = credentials
        with open(self.__root, 'w', encoding='utf-8') as file:
            file.write(json.dumps(self.__creds, indent=4))

    def export_credentials(self):
        try:
            os.popen(f'cp {self.__root} {self.__project + self.__platform + "gowork_secret_keys.json"}')
            print('Exported successfully')
        except:
            print('Exported failed')

    def import_credentials(self, path='.'):
        try:
            os.popen(f'cp {self.__project + self.__platform + path + self.__platform + "gowork_secret_keys.json"} {self.__root}')
            print('Imported successfully')
        except:
            print('File not found')

    def get_credentials(self) -> dict:
        self.__load()
        creds = {}
        for k in self.__creds.keys():
            for sk in self.__creds[k].keys():
                creds[k] = sk
        return creds

    def select(self, connector: str, name: str):
        name = name.lower()
        connector = connector.lower()

        self.__load()
        if connector in self.__creds:
            if name in self.__creds[connector]:
                return self.__creds[connector][name]
            else:
                print('Name connection not found')
        print('Connector not found')

    def __load(self):
        if os.path.exists(self.__root):
            with open(self.__root, 'r', encoding='utf-8') as file:
                self.__creds = json.load(file)

    def __encode(self, credentials: dict, keys: list):
        for key in keys:
            credentials[key] = {'encode': base64.b64encode(credentials[key].encode('utf-8')).decode('utf-8')}
        return credentials





from gowork.safe.configs import Credentials
from mongudb.sql import Transaction
from pyathena import connect
from pyathena.pandas.util import as_pandas
from sqlalchemy import create_engine
import pandas as pd
import glob
import pathlib
import base64
import platform


class AthenaGo:
    def __init__(self, name_connection: str):
        self.__decode(name_connection)
        self.__con = connect(aws_access_key_id=self.__cred['aws_access_key_id'],
                             aws_secret_access_key=self.__cred['aws_secret_access_key'],
                             s3_staging_dir=self.__cred['s3_staging_dir'],
                             region_name=self.__cred['region_name'])
        self.__cursor = self.__con.cursor()

    def read_sql(self, sql, verbose=False):
        if verbose:
            print(sql)
        return as_pandas(self.__cursor.execute(sql))

    def __decode(self, name: str):
        cred = Credentials().select('Athena', name)
        for k, d in cred.items():
            if type(d) == dict:
                cred[k] = base64.b64decode(d['encode'].encode('utf-8')).decode('utf-8')
        self.__cred = cred


class MysqlGo:
    def __init__(self, name_connection: str):
        self.__buildurl(name_connection)
        self.__engine()
        self.__pandas = pd

    def __buildurl(self, name):
        self.__decode(name)
        self.url = f'mysql+pymysql://{self.__cred["user"]}:{self.__cred["password"]}@{self.__cred["host"]}:{self.__cred["port"]}/{self.__cred["database"]}'

    def __engine(self):
        self.__engine = create_engine(self.url)

    def read_sql(self, sql, verbose=False):
        if verbose:
            print(sql)
        return self.__pandas.read_sql(sql, con=self.__engine)

    def __decode(self, name: str):
        cred = Credentials().select('MySQL', name)
        for k, d in cred.items():
            if type(d) == dict:
                cred[k] = base64.b64decode(d['encode'].encode('utf-8')).decode('utf-8')
        self.__cred = cred


class MongoGo:
    def __init__(self, name_connection: str):
        self.__decode(name_connection)

    def read_sql(self, sql, verbose=False):
        if verbose:
            print(sql)
        t = Transaction(self.__cred['database'], self.__cred['url']).query(sql).run()
        return pd.DataFrame([i for i in t.query(sql).run().iter()])

    def __decode(self, name: str):
        cred = Credentials().select('MongoDB', name)
        for k, d in cred.items():
            if type(d) == dict:
                cred[k] = base64.b64decode(d['encode'].encode('utf-8')).decode('utf-8')
        self.__cred = cred


class GoQuery:
    """This object load sql files and insert them into a dictionary to access via their key,
        which receives the same name as the file
    """
    def __init__(self, path: str):
        """
        :param path: Where your sql files are located
        """
        self.__platform = '\\' if platform.system().__str__() == 'Windows' else '/'
        self.__queries = {}
        self.path = path
        self.__root = pathlib.Path().resolve().__str__()
        self.__loadfiles()

    def use(self, query_name: str):
        """Select which query do you want use
        :param query_name: SQL file saved, without extension .sql
        """
        return self.__queries[query_name]

    def __percsign(self, query):
        """By default python uses %% to indicate use of %, so to handle with
        :param query: Any query type string
        """
        return query.replace('%%','%').replace('%','%%')

    def __loadfiles(self):
        """
        Internal method that go through all files in directory and insert into a dictionary
        :return: None
        """
        for path in glob.glob(f"{self.__root + self.__platform + self.path + self.__platform}*.sql"):
            self.__cachefile(path.split(self.__platform)[-1])

    def __cachefile(self, file: str):
        """
        Insert into dictionary
        :param file: File name
        :return: None
        """
        with open(self.__root + self.__platform + self.path + self.__platform + file, 'r', encoding='utf-8') as line:
            self.__queries[file.replace('.sql', '')] = self.__percsign(line.read())



import pandas as pd
import shutil
import os
import pathlib
import platform
import glob


class GoStorage:
    def __init__(self, frame: pd.DataFrame, part=1):
        self.__frame = frame
        self.__part = part
        self.__local = f'{pathlib.Path().resolve().__str__()}/temp'

    def __checkpart(self):
        return self.__frame.shape[0]%self.__part == 0

    def find_part(self, max, min):
        for i in range(max, min-1, -1):
            if self.__frame.shape[0]%i == 0:
                self.__size = self.__frame.shape[0]/i
                self.__part = i
                break
        return self

    def store(self):
        next = 0
        if os.path.exists(self.__local):
            self.clear()
            os.makedirs(self.__local)
        else:
            os.makedirs(self.__local)

        for i in range(self.__part):
            self.__frame.iloc[:, next:next+next].to_csv(f'{self.__local}/{i+1}_tmp.csv', index=False)
            next += self.__size
        return self


class GoFiles:

    def __init__(self, path: str, type='csv', sep=','):
        self.__platform = '\\' if platform.system().__str__() == 'Windows' else '/'
        self.__frames = {}
        self.path = path
        self.__root = pathlib.Path().resolve().__str__()
        self.__type = type
        self.__loadfiles()
        self.sep = sep

    def use(self, frame_name: str):
        return self.__frames[frame_name]

    def use_index(self, index: int):
        return self.__frames[list(self.__frames.keys())[index]]

    def __loadfiles(self):
        for path in glob.glob(f"{self.__root + self.__platform + self.path + self.__platform}*.{self.__type}"):
            self.__cachefile(path.split(self.__platform)[-1])

    def __cachefile(self, file: str):
        """
        Insert into dictionary
        :param file: File name
        :return: None
        """
        if self.__type == 'csv':
            result = pd.read_csv(self.__root + self.__platform + self.path + self.__platform + file, sep=self.sep)
        else:
            result = self.__root + self.__platform + self.path + self.__platform + file

        self.__frames[file.replace(f'.{self.__type}', '')] = result

    def clear(self):
        shutil.rmtree(self.__local)